<?php
/**
 * Stellar
 * Stellar is star rating system helps you rate and like articles on a website and keep track of results.
 * Exclusively on https://1.envato.market/stellar
 *
 * @encoding        UTF-8
 * @version         2.2.5
 * @copyright       (C) 2018 - 2024 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Stellar;

use Merkulove\Stellar\Unity\Settings;
use Merkulove\Stellar\Unity\TabAssignments;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/**
 * SINGLETON: Caster class contain main plugin logic.
 *
 * @since 1.0.0
 *
 **/
final class Caster {

	/**
	 * The one true Caster.
	 *
     * @since 1.0.0
     * @access private
	 * @var Caster
	 **/
	private static $instance;

    /**
     * Setup the plugin.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function setup() {

        /** Init Stellar. */
        PluginLogic::get_instance()->init_stellar();

        /** Creating stellar table. */
        $this->register_stellar_table();
        add_action( 'switch_blog', [$this, 'register_stellar_table' ] );

        /** Initialize Stellar logic. */
        PluginLogic::get_instance()->stellar_filter( Settings::get_instance()->options['position'] );

        /** Allow svg uploading. */
        SvgHelper::get_instance();

        /** Define hooks that runs on both the front-end as well as the dashboard. */
        $this->both_hooks();

        /** Define public hooks. */
        $this->public_hooks();

        /** Define admin hooks. */
        $this->admin_hooks();

    }

    /**
     * This method used in register_activation_hook
     * Everything written here will be done during plugin activation.
     *
     * @since 1.0.0
     * @access public
     */
    public function activation_hook() {

        $this->create_stellar_table();

        /** Prevent network activation */
        if ( is_multisite() && isset( $_GET[ 'networkwide' ] ) ) {

            wp_die( esc_html__( 'Network activation is not available for this plugin. Activate the plugin on separate blogs of your multi-site network.', 'stellar' ) );

        }

    }

    /**
     * Define hooks that runs on both the front-end as well as the dashboard.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function both_hooks() {

        /** Register Widget. */
        add_action( 'widgets_init', [$this, 'register_widget'] );

		/** Compatibility with third-part plugins */
		Compatibility::get_instance();

    }

    /**
     * Register all the hooks related to the public-facing functionality.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function public_hooks() {

        /** Work only on frontend area. */
        if ( is_admin() ) { return; }

        /** Add Schema markup */
        add_action( 'wp_head', [ $this, 'structured_data' ] );

        /** Load JS and CSS for Frontend Area. */
        FrontEnd::get_instance();

        Shortcode::get_instance();

    }

    /**
     * Register all of the hooks related to the admin area functionality.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function admin_hooks() {

        /** Work only in admin area. */
        if ( ! is_admin() ) { return; }

        /** Add additional scripts to admin area. */
        AdminScripts::get_instance();

        /** Add additional styles to admin area. */
        AdminStyles::get_instance();

		/** Ajax actions for admin area */
		Ajax::get_instance();

        /** Create widget on admin dashboard. */
        DashboardWidget::get_instance();

    }

    /**
     * Store our table name in $wpdb with correct prefix.
     *
     * @since 1.0.0
     * @access public
     **/
    public function register_stellar_table() {

        global $wpdb;

        $wpdb->stellar = "{$wpdb->prefix}stellar";

    }

    /**
     * Creating a stellar table.
     * id | stellar_id (Post ID) | rating | ip | guid | created | modified
     *
     * @since 1.0.0
     * @access public
     **/
    public function create_stellar_table() {

        global $wpdb;
        global $charset_collate;

        require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );

        /** Call this manually as we may have missed the init hook. */
        $this->register_stellar_table();

        /** @noinspection SqlDialectInspection */
        $sql_create_table = "
            CREATE TABLE " . $wpdb->stellar . " (
                id BIGINT(20) UNSIGNED NOT NULL AUTO_INCREMENT,
                stellar_id BIGINT(20) UNSIGNED NOT NULL DEFAULT '0',
                rating INT(3) UNSIGNED NOT NULL DEFAULT '0',
                length INT(3) UNSIGNED NOT NULL DEFAULT '0',
                ip varchar(64) CHARACTER SET utf8 NOT NULL DEFAULT '',
                guid varchar(64) CHARACTER SET utf8 NOT NULL DEFAULT '',
                created DATETIME DEFAULT NULL,
                modified DATETIME DEFAULT NULL,
                PRIMARY KEY (id)
            ) $charset_collate; ";

        dbDelta( $sql_create_table );

    }

    /**
     * Registers a WP_Widget widget.
     *
     * @return void
     * @since 1.1.6
     * @access public
     **/
    public function register_widget() {

        register_widget( Widget::class );

    }

	private function structured_data_conditions() {

		global $post;

		/** Get Plugin Settings. */
		$options = Settings::get_instance()->options;

		/** Checks if plugin should work on this page. */
		if( ! TabAssignments::get_instance()->display() ) {
			return apply_filters( 'stellar_schema_condition', false );
		}

		/** Exit on the Category and Tag pages */
		if ( is_archive() || is_404()||  is_category() || is_tag() || is_attachment() ) {
			return apply_filters( 'stellar_schema_condition', false );
		}

		/** Exit is Post type not listed in settings */
		if ( ! is_singular( $options[ 'cpt_support' ] ) ) {
			return apply_filters( 'stellar_schema_condition', false );
		}

		/** Show ld+json Markup if it is enabled in settings. */
		if ( ! ( 'on' === $options['advanced_markup'] && 'on' === $options['google_search_results'] ) ) {
			return apply_filters( 'stellar_schema_condition', false );
		}

		/**	If we in shortcode mode **/
		if ( 'shortcode' === $options['position'] && isset( $post->post_content ) ) {

			/** And current Page don't have shortcode. */
			if ( ! has_shortcode( $post->post_content, 'stellar' ) && isset( $options[ 'markup_everywhere' ] ) && 'off' === $options[ 'markup_everywhere' ] ) {
				return apply_filters( 'stellar_schema_condition', false );
			}

		}

		return apply_filters( 'stellar_schema_condition', true );

	}

    /**
     * Output ld+json Markup.
     *
     * @since 1.1.2
     * @access private
     * @return void
     **/
    public function structured_data() {

        if ( $this->structured_data_conditions() === false ) { return; }

        /** Get Plugin Settings. */
        $options = Settings::get_instance()->options;

        $id = get_post_field( 'ID' );
        if ( is_tax() ) {
			$title = htmlentities( get_queried_object()->name );
	    } else if ( is_category() || is_tag() ) {
			$title = htmlentities( single_cat_title( '', false ) );
        } else {
	        $title = htmlentities( get_post_field( 'post_title' ) );
        }
        $best = $options['number_icons'];
        $count = PluginLogic::get_instance()->number_of_voters( $id );
        $value = PluginLogic::get_instance()->get_rating_result( $id );

        /** We need at least one rating to show structured data. */
        if ( ! $value ) { return; }

        /** Get JSON+LD Markup from settings. */
        $json_ld = $options['json_ld'];

        /** Apply variables replacements. */
        $json_ld = str_replace( ['[title]', '[best]', '[count]', '[value]'], [$title, $best, $count, $value], $json_ld );

        /** Output ld+json Markup. */
        printf( '<script type="application/ld+json">%s</script>', apply_filters( 'stellar_structured_data', $json_ld ) );

    }

	/**
	 * Main Caster Instance.
	 * Insures that only one instance of Caster exists in memory at any one time.
	 *
	 * @static
     * @since 1.0.0
     * @access public
     *
	 * @return Caster
	 **/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {

			self::$instance = new self;

		}

		return self::$instance;

	}

}
