<?php
/**
 * Stellar
 * Stellar is star rating system helps you rate and like articles on a website and keep track of results.
 * Exclusively on https://1.envato.market/stellar
 *
 * @encoding        UTF-8
 * @version         2.2.5
 * @copyright       (C) 2018 - 2024 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Stellar;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

use Merkulove\Stellar\Unity\Plugin;
use Merkulove\Stellar\Unity\Settings;

/**
 * SINGLETON: Class used to create widgets on admin dashboard.
 *
 * @since 1.1.6
 **/
final class DashboardWidget {

	/**
	 * The one true DashboardWidget.
	 *
	 * @var DashboardWidget
	 * @since 1.1.6
	 **/
	private static $instance;

	/**
	 * Sets up a new DashboardWidget instance.
	 *
	 * @since 1.1.6
	 * @access public
	 **/
	private function __construct() {

		/** Add status widget to dashboard. */
		add_action( 'wp_dashboard_setup', [$this, 'dashboard_setup'] );

		/** Load JS and CSS for Backend Area. */
		$this->enqueue_backend();

	}

	/**
	 * Load JS and CSS for Backend Area.
	 *
	 * @since 1.1.6
	 * @access public
	 **/
	public function enqueue_backend() {

		/** Add admin styles. */
		add_action( 'admin_enqueue_scripts', [$this, 'admin_styles' ] );

	}

	/**
	 * Add CSS for admin area.
	 *
	 * @since 1.1.6
	 * @return void
	 **/
	public function admin_styles() {

	    /** Get current screen. */
		$screen = get_current_screen();
		if ( null === $screen ) { return; }

		/** Add styles only on dashboard page. */
		if ( 'dashboard' === $screen->base ) {

			wp_enqueue_style( 'mdp-stellar-dashboard', Plugin::get_url() . 'css/dashboard' . Plugin::get_suffix() . '.css', [], Plugin::get_version() );

		}

	}

	/**
	 * Fires after core widgets for the admin dashboard have been registered.
	 *
	 * @since 1.1.6
	 * @access public
	 *
	 * @return void
	 **/
	public function dashboard_setup() {

		wp_add_dashboard_widget( 'mdp_stellar_dashboard_widget', esc_html__( 'Stellar', 'stellar' ), [$this, 'dashboard_widget_statistics'] );

	}

	/**
	 * Show Stellar Statistic widget.
	 *
	 * @since 1.1.6
	 * @access public
	 *
	 * @return void
	 **/
	public function dashboard_widget_statistics() {

        $options = Settings::get_instance()->options;

		?><div class="mdp-stellar-wp-dashboard"><?php

            $this->render_overall_rating();

            if ( 'hide' !== $options[ 'results_admin' ] ) {

	            foreach ( Settings::get_instance()->options[ 'cpt_support' ] as $post_type_name ) {

		            /** Get a post type object by name. */
		            $post_type = get_post_type_object( $post_type_name );
		            if ( null === $post_type ) { continue; }

		            /** Render list of 5 top-rated posts by post_type. */
		            $this->render_top_rated( $post_type );

	            }

            }

        ?></div><?php

	}

    /**
     * Render list of 5 top rated posts by post_type.
     *
     * @param object $post_type
     * @param string $title
     * @param integer $limit
     *
     * @since  1.1.6
     * @access public
     *
     * @return void
     **/
    public function render_top_rated( $post_type, $title = '', $limit = 5 ) {
        global $wpdb;

        /** Get plugin settings. */
        $options = Settings::get_instance()->options;

        /**
         * Get top liked 5 posts of selected $post_type.
         *
         * @noinspection SqlDialectInspection
         **/
        $posts = $wpdb->get_results(
            $wpdb->prepare("
                SELECT $wpdb->stellar.stellar_id as post_id, ROUND( SUM( $wpdb->stellar.rating / $wpdb->stellar.length ) * %d ) AS rating, COUNT( $wpdb->stellar.stellar_id ) as total_votes
                FROM $wpdb->stellar
                INNER JOIN $wpdb->posts ON $wpdb->stellar.stellar_id = $wpdb->posts.ID
                WHERE $wpdb->posts.post_type = %s
                GROUP BY $wpdb->stellar.stellar_id
                ORDER BY rating DESC
                LIMIT %d;", [ $options['number_icons'], $post_type->name, 1000 ] )
        );

        /** No posts with stellar. Nothing to show. */
        if ( empty( $posts ) ) { return; }

        $posts_order = PluginLogic::get_instance()->calculate_rating( $posts ); // Calculate and add page rating to array
        arsort( $posts_order, SORT_NUMERIC ); // Sort by rating
        $posts_order = array_slice( $posts_order, 0, $limit, true ); // Crop by top value

        /** Header "Top Rated {POST_TYPE}:" */
        if ( $title ) {

            ?><h3 class="mdp-stellar-cpt-title"><?php esc_html_e( $title ); ?></h3><?php

        } elseif ( false !== $title ) {
            ?>
            <h3 class="mdp-stellar-cpt-title">
                <strong>
                    <?php
                    esc_html_e( 'Top Rated ', 'stellar' );
                    esc_html_e( $post_type->label );
                    ?>
                </strong>
            </h3>
            <?php
	        $this->render_overall_rating( $post_type );
        }
        ?>
        <div class="mdp-stellar-list-box">
            <ul class="mdp-stellar-list" role="list">
                <?php foreach ( $posts_order as $post_id => $post_rating ) :
                    $count = PluginLogic::get_instance()->number_of_voters( $post_id );
                    ?>
                    <li class="mdp-stellar-list-item">
                        <a class="mdp-stellar-post-link" target="_blank" href="<?php echo get_permalink( $post_id ); ?>"><?php echo get_the_title( $post_id ); ?></a>
                        <span class="mpd-align">
                            <span class="mdp-stellar-rating" title="<?php esc_attr_e( 'Page Rating', 'stellar' ); ?>">
                                <span class="dashicons-star-filled mdp-dashicons"></span>
                                <?php esc_attr_e( $post_rating ); ?>
                            </span>
                            <?php if ( 'votes' === $options[ 'results_admin' ] ) : ?>
                            <span class="mdp-stellar-count" title="<?php esc_attr_e( 'Voted', 'stellar' ); ?>">
                                <span class="dashicons-admin-users mdp-dashicons"></span>
                                <?php esc_attr_e( $count ); ?>
                            </span>
                            <?php endif; ?>
                        </span>
                    </li>
                <?php endforeach; ?>
            </ul>
        </div>
        <?php

    }


	/**
     * Render overall rating for website or single post type
     *
	 */
	public function render_overall_rating( $post_type = [] ) {

		global $wpdb;

		/** Get plugin settings. */
		$options = Settings::get_instance()->options;
        $posts_order = array();

		/**
		 * Get top liked 5 posts of selected $post_type.
		 *
		 * @noinspection SqlDialectInspection
		 **/
        if ( ! empty( $post_type ) ) {

	        $posts = $wpdb->get_results(
		        $wpdb->prepare("
                SELECT $wpdb->stellar.stellar_id as post_id, ROUND( SUM( $wpdb->stellar.rating / $wpdb->stellar.length ) * %d ) AS rating, COUNT( $wpdb->stellar.stellar_id ) as total_votes
                FROM $wpdb->stellar
                INNER JOIN $wpdb->posts ON $wpdb->stellar.stellar_id = $wpdb->posts.ID
                WHERE $wpdb->posts.post_type = %s
                GROUP BY $wpdb->stellar.stellar_id
                ORDER BY rating DESC
                LIMIT 1000;", [ $options['number_icons'], $post_type->name ] )
	        );

        } else {

	        $posts = $wpdb->get_results(
		        $wpdb->prepare("
                SELECT $wpdb->stellar.stellar_id as post_id, ROUND( SUM( $wpdb->stellar.rating / $wpdb->stellar.length ) * %d ) AS rating, COUNT( $wpdb->stellar.stellar_id ) as total_votes
                FROM $wpdb->stellar
                INNER JOIN $wpdb->posts ON $wpdb->stellar.stellar_id = $wpdb->posts.ID
                GROUP BY $wpdb->stellar.stellar_id
                ORDER BY rating DESC
                LIMIT 1000;", [ $options['number_icons'] ] )
	        );

        }

		/** No posts with stellar. Nothing to show. */
		if ( empty( $posts ) ) { return; }

        $posts_order = PluginLogic::get_instance()->calculate_rating( $posts ); // Calculate and add page rating to array
        $overall = number_format(array_sum( $posts_order )/count( $posts_order ), 2 );

		/** Header "Top Rated {POST_TYPE}:" */
		if ( !empty( $post_type ) ) {

			?><p><?php
                esc_html_e( 'Average ', 'stellar' );
                esc_html_e( $post_type->label );
                esc_html_e( ' Rating: ', 'stellar' );
                echo esc_attr( $overall );
            ?></p><?php

		} else {
			?>
            <h3 class="mdp-stellar-cpt-title">
                <strong>
					<?php
					esc_html_e( 'Overall Website Rating: ', 'stellar' );
                    ?><span class="dashicons dashicons-star-filled mdp-dashicons"></span><?php
					echo esc_attr( $overall );
					?>
                </strong>
            </h3>
			<?php
		}

	}

	/**
	 * Main DashboardWidget Instance.
	 *
	 * Insures that only one instance of DashboardWidget exists in memory at any one time.
	 *
	 * @static
	 * @return DashboardWidget
	 * @since 1.1.6
	 **/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {

			self::$instance = new self;

		}

		return self::$instance;

	}

} // End Class DashboardWidget.
