<?php

/**
 * Stellar
 * Stellar is star rating system helps you rate and like articles on a website and keep track of results.
 * Exclusively on https://1.envato.market/stellar
 *
 * @encoding        UTF-8
 * @version         2.2.5
 * @copyright       (C) 2018 - 2024 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Stellar;

use WP_Post;
use WP_Query;
use Merkulove\Stellar\Unity\Plugin;
use Merkulove\Stellar\Unity\Settings;
use Merkulove\Stellar\Unity\TabAssignments;

/**
 * Class PluginLogic
 * @package Merkulove\Stellar
 */
final class PluginLogic {

	/**
	 * The one true PluginLogic.
	 *
	 * @var PluginLogic
	 * @since 1.0.0
	 **/
	private static $instance;

	/**
	 * Sets up a new PluginLogics instance.
	 *
	 * @since 1.0.0
	 * @access private
	 **/
	private function __construct() {

		/** Reset stellar.  */
		if ( defined('DOING_AJAX') ) {
			add_action( 'wp_ajax_reset_stellar', [$this, 'reset_stellar' ] );
        }

	}

	/**
	 * Remove all Stellar data from DataBase.
	 *
	 * @return void
	 * @since 1.0.0
	 * @access public
	 */
	public function reset_stellar() {
		global $wpdb;

		/** Check nonce for security. */
		check_ajax_referer( 'reset_stellar', 'nonce' );

		/** Do we need to do a full reset? */
		if ( empty( $_POST['doReset'] ) ) {
			wp_die( 'Wrong parameter value.' );
		}

        /** Remove stellar data from table. */
		$sql = "TRUNCATE TABLE " . $wpdb->prefix . "stellar";
		$res = $wpdb->get_results(
			$wpdb->prepare( $sql )
		);

        /** Mark time when we make last reset. */
        update_option( 'mdp_stellar_reset_timestamp', date('U' ) );

		/** Return JSON result. */
		if ( $res ) {
			echo json_encode( true );
		} else {
			echo json_encode( false );
		}

		/** Exit. */
		wp_die();
	}

	/**
     * Init Stellar.
     *
	 * @since 1.0.0
	 * @access public
	 */
	public function init_stellar() {

		/** Clear likes on remove post/page. */
		add_action( 'before_delete_post', [$this, 'before_delete_post'] );

		/** Add action to AJAX process like. */
		add_action( 'wp_ajax_process_stellar', [$this, 'process_stellar' ] );
		add_action( 'wp_ajax_nopriv_process_stellar', [$this, 'process_stellar' ] );

		/** We return the page rank after the user has voted. */
		add_action( 'wp_ajax_stellar_page_rating', [$this, 'stellar_page_rating' ] );
		add_action( 'wp_ajax_nopriv_stellar_page_rating', [$this, 'stellar_page_rating' ] );

		/** Add action to AJAX check like on load. */
		add_action( 'wp_ajax_stellar_get_rating', [ Ajax::get_instance(), 'get_rating' ] );
		add_action( 'wp_ajax_nopriv_stellar_get_rating', [ Ajax::get_instance(), 'get_rating' ] );

		/** Add Stellar column to Selected Posts list. */
        AdminPostTable::get_instance();

		/**
		 * The filter posts_results is executed just after the query
		 * was executed. We'll use it as a after_get_posts-action.
		 **/
		add_filter( 'posts_results', [$this, 'after_set_post_query'], 10, 2 );

	}

	/**
	 * The filter posts_results is executed just after the query
	 * was executed. We'll use it as a after_get_posts-action.
	 *
	 * @param WP_Post[] $posts - Array of post objects.
	 * @param WP_Query  $query - The WP_Query instance (passed by reference).
	 *
	 * @since  1.0.0
	 * @access public
	 * @return mixed
	 **/
	public function after_set_post_query( $posts, $query ) {

	    /** Work only in admin area. */
		if ( ! is_admin() ) { return $posts; }

		/** Work only with stellar column sort. */
		$order_by = $query->get( 'orderby');
		if ( 'stellar-results' !== $order_by ) { return $posts; }

		/** Get order: ASC or DESC. */
		$order = $query->get( 'order');

		/** Do sort by float rating field. */
		usort( $posts, function( $a, $b ) use ( $order ) {

			$result = 0;

			$a_rating = $this->get_rating_result( $a->ID );
			$b_rating = $this->get_rating_result( $b->ID );

			if ( $a_rating > $b_rating ) {

			    if ( 'ASC' === $order ) {
				    $result = 1;
                } else {
				    $result = -1;
                }


			} elseif ( $a_rating < $b_rating ) {

				if ( 'ASC' === $order ) {
					$result = -1;
				} else {
					$result = 1;
				}

			}

			return $result;

		} );

	    return $posts;

    }

	/**
	 * AJAX Page Rating Stellar.
     * We return the page rank after the user has voted.
     *
	 * @since 1.0.0
	 * @access public
	 **/
	public function stellar_page_rating() {

		$statusRequest = (bool) $_POST['page_rating'];
		$stellar_id = (int) $_POST['stellar_id'];
		$page_rating = $this->get_rating_result( $stellar_id );
		$this->number_of_voters($stellar_id) ? $count_voters = $this->number_of_voters($stellar_id) : $count_voters = 0;

		if ( $statusRequest ) {
			/** Exceeded the limit of votes from one IP. */
			echo json_encode( ['status' => 0, 'votes' => esc_attr__( $count_voters ), 'page_rating' => esc_attr__( $page_rating )] );

			/** Required to terminate immediately and return a proper response. */
			wp_die();
        }
	}

	/**
	 * AJAX Process Stellar.
	 * Users can like 10 times from 1 IP in 24 hours.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function process_stellar() {

		$is_new = (bool) $_POST['new_stellar'];
		$stellar_id = (int) $_POST['stellar_id'];
		$rating = (int) $_POST['rating'];
		$length = (int) $_POST['length'];
		$user_ip = sanitize_text_field( $this->get_ip() );
		$guid = sanitize_text_field( $_POST['guid'] );
		$created = gmdate( 'Y-m-d H:i:s' );
		$modified = gmdate( 'Y-m-d H:i:s' );
		$this->number_of_voters($stellar_id) ? $count_voters = $this->number_of_voters($stellar_id) : $count_voters = 0;
		$new_result = $this->get_rating_result($stellar_id);
        $page_rating = $this->get_rating_result( $stellar_id );

        /** New Like. */
		if ( $is_new ) {

			/** Check limits to like from this IP. */
			if ( $this->check_stellar_limits( $stellar_id, $user_ip ) ) {

				/** Insert like in table. */
				$this->insert_stellar( $stellar_id, $rating, $length, $user_ip, $guid, $created, $modified );

			} else {

				echo json_encode(
					[
						'status' => 0,
						'is_new' => 'insert',
						'votes' => esc_attr__( $count_voters ),
						'page_rating' => esc_attr__( $page_rating ),
						'message' => esc_html__( 'Insert  Exceeded the limit of votes from one IP.', 'stellar' ),
						'new_result' => $new_result
					]
				);

				/** Required to terminate immediately and return a proper response. */
				wp_die();
			}
			/** User already Liked. */
		} else {

            /** Update previous like. */
            $this->update_stellar( $stellar_id, $rating, $length, $user_ip, $guid, $modified );

            echo json_encode(
				[
					'status' => 0,
					'is_new' => 'update',
					'votes' => esc_attr__( $count_voters ),
					'page_rating' => esc_attr__( $page_rating ),
					'new_result' => $new_result
				]
            );

            /** Required to terminate immediately and return a proper response. */
            wp_die();

		}

		/** All OK. */
		echo json_encode( ['status' => 1, 'is_new' => $is_new, 'votes' => esc_attr__( $count_voters ), 'page_rating' => esc_attr__( $page_rating ), 'new_result' => $new_result] );

		/** Required to terminate immediately and return a proper response. */
		wp_die();

	}

	/**
	 * Update existing like value.
	 *
	 * @param $stellar_id
	 * @param $rating
	 * @param $length
	 * @param $user_ip
	 * @param $guid
	 * @param $modified
	 *
	 * @since 1.0.0
	 * @return false|int
	 **/
	public function update_stellar( $stellar_id, $rating, $length, $user_ip, $guid, $modified ) {
		global $wpdb;

		/** @noinspection PhpUnnecessaryLocalVariableInspection */
		$res = $wpdb->update(
			$wpdb->stellar, ['rating' => $rating, 'length' => $length, 'ip' => $user_ip, 'modified' => $modified], ['stellar_id' => $stellar_id, 'guid' => $guid], ['%d', '%d', '%s', '%s'], ['%d', '%s']
		);

		return $res;
	}

	/**
	 * Insert new like value.
	 *
	 * @param $stellar_id
	 * @param $rating
	 * @param $length
	 * @param $user_ip
	 * @param $guid
	 * @param $created
	 * @param $modified
	 *
	 * @since 1.0.0
	 * @return int
	 **/
	public function insert_stellar( $stellar_id, $rating, $length, $user_ip, $guid, $created, $modified ) {
		global $wpdb;

		$wpdb->insert(
			$wpdb->stellar, ['stellar_id' => $stellar_id, 'rating' => $rating, 'length' => $length, 'ip' => $user_ip, 'guid' => $guid, 'created' => $created, 'modified' => $modified], ['%d', '%d', '%d', '%s', '%s', '%s', '%s']
		);

		return $wpdb->insert_id;
	}

	/**
	 * Users can like 10 times from 1 IP in 24 hours.
	 *
	 * @param $stellar_id
	 * @param string $ip
	 * @return bool
	 * @since 1.0.0
	 **/
	public function check_stellar_limits( $stellar_id, $ip ) {
		global $wpdb;

		/** @noinspection SqlDialectInspection */
		$res = $wpdb->get_results(
			$wpdb->prepare("
                SELECT * FROM $wpdb->stellar
                WHERE ($wpdb->stellar.modified > DATE_SUB(NOW(), INTERVAL 24 HOUR)) AND stellar_id = %d AND ip = %s",
				[$stellar_id, $ip]
			)
		);

		$voting_limit = Settings::get_instance()->options['limit_voting'];

		return ! ( count( $res ) >= $voting_limit );

	}

	/**
	 * Get user IP.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function get_ip() {

		if (!empty($_SERVER['HTTP_CLIENT_IP'])) {
			$ip = $_SERVER['HTTP_CLIENT_IP'];
		} elseif (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
			$ip = $_SERVER['HTTP_X_FORWARDED_FOR']; //to check ip passed from proxy
		} else {
			$ip = $_SERVER['REMOTE_ADDR'];
		}

		/** Add some validation. */
		if ( ! filter_var( $ip, FILTER_VALIDATE_IP ) ) {
			$ip = "UNKNOWN";
		}

		return $ip;
	}

	/**
	 * Clear likes after remove post/page.
	 *
	 * @param $post_id
	 *
	 * @return false|int|void
	 * @since 1.0.0
	 * @access public
	 **/
	public function before_delete_post( $post_id ) {

		global $wpdb;

		/** Work only with Post and Page type. */
		if ( ! in_array( get_post_type( $post_id ), Settings::get_instance()->options['cpt_support'], false ) ) { return; }

		return $wpdb->delete( $wpdb->stellar, [ 'stellar_id' => $post_id] );

	}

	/**
     * Add filter
	 *
     * @param $position - position stellar
	 * @since 1.0.0
	 * @access public
	 */
	public function stellar_filter( $position ) {

		/** Add stellar before/after Title. */
        if ( in_array( $position, ['after-title', 'before-title'] ) ) {
	        add_filter( 'the_title', [$this, 'add_to_title'] );
        }

		/** Add stellar before/after Content. */
        if ( in_array( $position, ['before-content', 'after-content'] ) ) {
			add_filter( 'the_content', [$this, 'add_to_content'] );
		}

	}

	/**
     *  We calculate the rating depending on its length.
     *
	 * @param $id
	 * @since 1.0.0
	 * @return float|int
	 **/
	public function get_rating_result( $id ) {

		$number_icons = Settings::get_instance()->options['number_icons'];

		/** Prepare values for calculation. */
		$result_value = $this->get_single_result_value( $id );
		if ( ! $result_value ) { $result_value = 0; }

		$count_voters = $this->number_of_voters( $id );
		if ( ! $count_voters ) { $count_voters = 0; }

		$result = 0;
		if ( 0 !== $count_voters ) {

			$result = $result_value / $count_voters;
			$result *= $number_icons;
			$result = round( $result, 2 );

		}

		return $result;

	}

	/**
	 * Show Schema markup only once per page.
	 *
	 * @since 1.1.0
	 * @access public
	 **/
	public function is_schema() {

		/** Get plugin settings. */
		$options = Settings::get_instance()->options;

		/** If true then show Schema markup. */
		$isSchema = $options[ 'google_search_results' ] === 'on';

		/**
		 * Set false If we use Advanced Schema Markup.
         * @see Stellar->structured_data() for details.
		 **/
		if ( 'on' === $options['advanced_markup'] && 'on' === $options['google_search_results'] ) { return false; }

		if ( $isSchema ) {

			/** Show Schema markup only once. */
			static $already_run = false;

			if ( $already_run === true ) {
				$isSchema = false;
			}

			$already_run = true;

		}

		return $isSchema;
	}

	/**
	 * Return Stellar code.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function get_stellar( $atts = [] ) {

        $id = isset( $atts['id'] ) ? $atts[ 'id' ] : get_the_ID();
        $options = Settings::get_instance()->options; // Stellar settings
        $layout = $options[ 'layout' ]; // Stellar layout
        $number_icons = $options[ 'number_icons' ]; // Rating capacity
        $result = strval( $this->get_rating_result( $id ) ); // Rating value
        $count_voters = $this->number_of_voters( $id ); // Votes count
        $isSchema = $this->is_schema(); // If true then show Schema markup
		$refresh = isset( $options[ 'refresh' ] ) && $options[ 'refresh' ] === 'on' ? 'true' : 'false'; // Refresh page after loading

        $options[ 'results_text' ] ? $results_test = $options[ 'results_text' ] : $results_test = ''; // Rating message
        $options[ 'votes_text' ] ? $votes_text = $options[ 'votes_text' ] : $votes_text = ''; // Votes count message

        /** Prepare variables */
        if ( $count_voters < 0 ) { $count_voters = 0; }
        if ( ! $count_voters ) { $isSchema = false; }

        /** Prepare classes. */
		$classes = [];
		$classes[] = $options[ 'position' ]; // Position.
        $classes[] = $options[ 'layout' ]; // Layout.

        /** Prepare values for calculation. */
        $this->get_single_result_value( $id ) ? $result_value = $this->get_single_result_value( $id ) : $result_value = 0;

        ob_start();

    ?>
        <!-- Stellar WordPress Plugin - Begin -->
        <div id="mdp-stellar-<?php echo esc_attr( $id ); ?>"
             class="mdp-stellar-box <?php echo esc_attr( implode( ' ', $classes ) ); ?>"
             data-rating="<?php esc_attr_e( $result ); ?>"
             data-voted="<?php esc_attr_e( $count_voters ); ?>"
             data-capacity="<?php esc_attr_e( $number_icons ); ?>"
             data-round="<?php esc_attr_e( $options[ 'round' ] ); ?>"
             data-secrecy="<?php esc_attr_e( $options[ 'secrecy' ] ); ?>"
             data-your-rating="<?php esc_attr_e( $options[ 'latest' ] ); ?>"
             data-refresh="<?php esc_attr_e( $refresh ); ?>"
	        <?php echo ( $isSchema ) ? ' itemtype="http://schema.org/Article"' : ''; ?>
        >

            <div class="mdp-stellar">

                <div class="mdp-stellar-description">
		            <?php  echo wp_kses_post( $options['description'] ); ?>
                </div>

                <div class="mdp-stellar-buttons"
					<?php echo ( $isSchema ) ? '  itemprop="aggregateRating" itemscope="" itemtype="http://schema.org/AggregateRating"' : ''; ?>
                >
					<?php if( $isSchema ): ?>
                        <div itemprop="itemReviewed" itemscope="" itemtype="https://schema.org/Organization" class="mdp-hidden">
                            <span itemprop="name" style="display: none"><?php echo get_the_title(  $id ); ?></span>
                            <span itemprop="description" style="display: none"><?php echo get_the_excerpt( $id ); ?></span>
                        </div>
                        <span itemprop="ratingValue" class="mdp-hidden"><?php esc_html_e( $result ); ?></span>
                        <span itemprop="worstRating" class="mdp-hidden">1</span>
                        <span itemprop="bestRating" class="mdp-hidden"><?php esc_html_e($number_icons); ?></span>
                        <span itemprop="ratingCount" class="mdp-hidden"><?php esc_html_e( $count_voters ) ; ?></span>
					<?php endif; ?>

                    <span class="mdp-hidden mdp-best-rating" data-rating="<?php esc_html_e( $number_icons ); ?>"></span>

                    <div id="mdp-box-icon" class="mdp-box-icon mdp-<?php echo esc_attr( $layout ) ?>" data-rating="<?php esc_html_e($number_icons); ?>">

                        <div class="mdp-stellar-stars">
                        <?php for ( $i = 1; $i <= $number_icons; $i++ ): ?>
                            <i data-id="<?php echo esc_attr($i); ?>">
                                <?php echo  $this->svg() ?>
                                <div class="mdp-stellar-segment"><?php echo  $this->svg() ?></div>
                            </i>
                        <?php endfor ?>
                        </div>

	                    <?php if ( $options[ 'valuation' ] !== 'hide' ): ?>
                        <span class="mdp-stellar-<?php esc_attr_e( $options[ 'valuation' ] ) ?>">
                            <span class="mdp-rating-res">0</span>
                            <span>/</span>
                            <span class="mdp-max-rating"><?php esc_html_e($number_icons); ?></span>
                        </span>
	                    <?php endif; ?>

                        <?php if ( $options[ 'results' ] === 'before' || $options[ 'results' ] === 'after' ): ?>
                        <span class="mdp-stellar-results-<?php esc_attr_e( $options[ 'results' ] ) ?>">
                            <span><?php esc_html_e( $results_test, 'stellar' );?></span>
                            <span class="mdp-stellar-results"><?php esc_html_e( $result ); ?></span>
                        </span>
                        <?php endif; ?>

                        <?php if ( $options[ 'votes' ] === 'before' || $options[ 'votes' ] === 'after' ): ?>
                        <span class="mdp-stellar-votes-<?php esc_attr_e( $options[ 'votes' ] ) ?>">
                            <span><?php esc_html_e( $votes_text, 'stellar' );?></span>
                            <span class="mdp-stellar-votes"><?php esc_html_e( $count_voters ); ?></span>
                        </span>
                        <?php endif; ?>

                    </div>

                    <p class="mdp-stellar-your-rating mdp-hidden"><?php esc_html_e( $options[ 'latest_text' ], 'stellar' ); ?> <span></span></p>

                </div>
            </div>
        </div>
        <!-- Stellar WordPress Plugin - End -->
		<?php
		return ob_get_clean();
	}

	/**
	 * Return single voting result value.
	 *
	 * @param $id
	 *
	 * @since  1.0.0
	 * @access public
	 * @return false|string
	 **/
	public function get_single_result_value( $id ) {

		/** Get likes data from DB.*/
		return $this->get_stellar_data( $id );
	}

	/**
	 * Add stellar before/after Title.
	 *
	 * @param $title
	 *
	 * @return string
	 * @since 1.0.0
	 * @access public
     *
     * @return string
	 */
	public function add_to_title( $title ) {

		/** Checks if plugin should work on this page. */
		if ( ! TabAssignments::get_instance()->display() ) { return $title; }

		/** Check if we are in the loop and work only with Pages and Posts. */
		if ( ! (  in_the_loop() && is_singular( Settings::get_instance()->options['cpt_support'] ) ) ) { return $title; }

		/** Run only once. */
		static $already_run = false;
		if ( $already_run === true ) { return $title; }
		$already_run = true;

		$stellar = $this->get_stellar();
		if ( Settings::get_instance()->options['position'] === 'before-title' ) {

			return $stellar . $title;

		}

		if ( Settings::get_instance()->options['position'] === 'after-title' ) {

			return $title . $stellar;

		}

		return $title;

	}

	/**
	 * Add stellar before/after Content.
	 *
	 * @param $content
	 *
	 * @return string
	 * @since 1.0.0
	 * @access public
     *
     * @return string
	 */
	public function add_to_content( $content ) {

		/** Checks if plugin should work on this page. */
		if ( ! TabAssignments::get_instance()->display() ) { return $content . ''; }

		/** Check if we are in the loop and work only with Pages and Posts. */
		if ( ! ( in_the_loop() && is_singular( Settings::get_instance()->options['cpt_support'] ) ) ) { return $content; }

		/** Run only Once. */
		static $already_run = false;
		if ( $already_run === true ) { return $content; }
		$already_run = true;

		$stellar = $this->get_stellar();
		if ( Settings::get_instance()->options['position'] === 'before-content' ) {

			return $stellar . $content;

		}

		if ( Settings::get_instance()->options['position'] === 'after-content' ) {

			return $content . $stellar;

		}

		return $content;

	}

	/**
	 * Get stellar data.
	 *
	 * @param $stellar_id
	 *
	 * @return int
	 **/
	public function get_stellar_data( $stellar_id ) {

		global $wpdb;

		/** @noinspection SqlDialectInspection */
		$result = $wpdb->get_results(
			$wpdb->prepare("
                SELECT $wpdb->stellar.stellar_id, SUM( $wpdb->stellar.rating / $wpdb->stellar.length ) AS cnt_rating
                FROM $wpdb->stellar
                WHERE $wpdb->stellar.stellar_id = %d
                GROUP BY $wpdb->stellar.stellar_id", [$stellar_id] )
		);


		if ( isset( $result[0]->cnt_rating )  ) {
			$res = $result[0]->cnt_rating;
		} else {
			$res = 0;
        }

		return $res;
	}

	public function number_of_voters( $stellar_id ) {

		global $wpdb;

		/** @noinspection SqlDialectInspection */
		$db_res = $wpdb->get_results(
			$wpdb->prepare("
                SELECT COUNT(*)  AS cnt_rating
                FROM $wpdb->stellar
                WHERE $wpdb->stellar.stellar_id = %d
                ", [$stellar_id] )
		);

		$count = $db_res[0]->cnt_rating;
		if ( $count < 0 ) { $count = 0; }

		return $count;
	}

	/**
     * Calculate rating for each post in object from wpdb
     *
	 * @param $raw_posts
	 *
	 * @return array
	 */
    public function calculate_rating( $raw_posts ) {

        $posts = [];

	    for ( $i = 0; $i < count( $raw_posts ) ; $i++ ) {

		    $this_post_rating = PluginLogic::get_instance()->get_rating_result( $raw_posts[ $i ]->post_id );
		    $this_post_id = $raw_posts[ $i ]->post_id;

		    if ( $this_post_rating > 0 ) {
			    $posts[ "{$this_post_id}" ] = $this_post_rating;
		    }

	    }

        return $posts;

    }

    /**
     * Return svg icon
     *
     * @return string - SVG icon
     */
    public function svg() {

        /** Prepare icon */
        $icon = Settings::get_instance()->options[ 'icon' ]; // Rating Icon
        $icon = is_numeric( $icon ) ? get_attached_file( $icon ) : Plugin::get_path() . 'images/mdc-icons/' . $icon;
        $svg_icon = file_get_contents( $icon, false, stream_context_create( [ 'http' => [ 'ignore_errors' => true ] ] ) ); // Rating SVG icon code

        /** Escaping SVG with KSES. */
        $allowed_tags = array_merge( wp_kses_allowed_html( 'post' ), [
            'svg' => [
                'class' => true,
                'aria-hidden' => true,
                'aria-labelledby' => true,
                'role' => true,
                'xmlns' => true,
                'width' => true,
                'height' => true,
                'viewbox' => true, // <= Must be lower case!
            ],
            'g' => [ 'fill' => true ],
            'title' => [ 'title' => true ],
            'path' => [ 'd' => true, 'fill' => true, ],
        ] );

        return wp_kses( $svg_icon, $allowed_tags );
    }

	/**
	 * Main PluginLogic Instance.
	 *
	 * Insures that only one instance of Plugin Logic exists in memory at any one time.
	 *
	 * @static
	 * @since 1.0.0
     *
	 * @return PluginLogic
	 **/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {

			self::$instance = new self;

		}

		return self::$instance;

	}

}
